package com.flow.common.core.exception.handler;

import com.flow.common.core.exception.BaseException;
import com.flow.common.core.model.Result;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.Ordered;
import org.springframework.core.annotation.Order;
import org.springframework.http.HttpStatus;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.web.HttpMediaTypeNotSupportedException;
import org.springframework.web.HttpRequestMethodNotSupportedException;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import javax.servlet.http.HttpServletResponse;
import java.util.Objects;

/**
 * @author caixiaofeng
 * @version 1.0
 * @Description: <<全局异常捕获>>
 * @company 个人学习
 * @create 2020-11-04 10:30
 */
@Slf4j
@RestControllerAdvice
@Order(value = Ordered.HIGHEST_PRECEDENCE)
public class BaseExceptionHandler {

    /**
     * Http请求方法不受支持异常
     * @param e
     * @return
     */
    @ExceptionHandler(value = HttpRequestMethodNotSupportedException.class)
    @ResponseStatus(HttpStatus.INTERNAL_SERVER_ERROR)
    public Result handleHttpRequestMethodNotSupportedException(HttpRequestMethodNotSupportedException e) {
        String message = "不支持Http请求方法";
        log.error(message);
        return Result.error(message);
    }

    @ExceptionHandler(value = HttpMediaTypeNotSupportedException.class)
    @ResponseStatus(HttpStatus.INTERNAL_SERVER_ERROR)
    public Result handleHttpMediaTypeNotSupportedException(HttpMediaTypeNotSupportedException e) {
        String message = "Http不支持的媒体类型";
        log.error(message);
        return Result.error(message);
    }

    /**
     * 处理所有不可知的异常
     */
    @ExceptionHandler(Throwable.class)
    @ResponseStatus(HttpStatus.INTERNAL_SERVER_ERROR)
    public Result handleException(Throwable e){
        e.printStackTrace();
        return Result.error("系统内部异常，请联系管理员");

    }

    /**
     * 自定义异常处理
     * @param e
     * @return
     */
    @ExceptionHandler(BaseException.class)
    @ResponseStatus(HttpStatus.BAD_REQUEST)
    public Result baseHandleException(BaseException e){
        e.printStackTrace();
        return Result.error(e.getMessage());
    }

    @ExceptionHandler(AccessDeniedException.class)
    @ResponseStatus(HttpStatus.BAD_REQUEST)
    public Result accessDeniedException(AccessDeniedException e){
        e.printStackTrace();
        return Result.error(e.getMessage());
    }


    /**
     * 参数异常拦截
     *
     * @param response
     * @param e
     * @return
     */
    @ExceptionHandler(value = MethodArgumentNotValidException.class)
    @ResponseStatus(HttpStatus.BAD_REQUEST)
    public Result nullPointerExceptionHandler(HttpServletResponse response, MethodArgumentNotValidException e) {
        String field = Objects.requireNonNull(e.getBindingResult().getFieldError()).getField();
        String defaultMessage = e.getBindingResult().getFieldError().getDefaultMessage();
        return Result.error(field + ":" + defaultMessage);
    }

    /**
     * Assert错误拦截
     *
     * @param e
     * @return
     */
    @ExceptionHandler(value = IllegalArgumentException.class)
    @ResponseStatus(HttpStatus.BAD_REQUEST)
    public Result IllegalArgumentException(IllegalArgumentException e) {
        return Result.error(e.getMessage());
    }
}
