package com.flow.common.oauth2.configure;

import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.data.redis.connection.RedisConnectionFactory;
import org.springframework.http.HttpMethod;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.AuthenticationProvider;
import org.springframework.security.authentication.ProviderManager;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.userdetails.UserDetailsByNameServiceWrapper;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.oauth2.common.DefaultOAuth2AccessToken;
import org.springframework.security.oauth2.common.OAuth2AccessToken;
import org.springframework.security.oauth2.config.annotation.configurers.ClientDetailsServiceConfigurer;
import org.springframework.security.oauth2.config.annotation.web.configuration.AuthorizationServerConfigurerAdapter;
import org.springframework.security.oauth2.config.annotation.web.configuration.EnableAuthorizationServer;
import org.springframework.security.oauth2.config.annotation.web.configurers.AuthorizationServerEndpointsConfigurer;
import org.springframework.security.oauth2.config.annotation.web.configurers.AuthorizationServerSecurityConfigurer;
import org.springframework.security.oauth2.provider.ClientDetailsService;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.security.oauth2.provider.OAuth2RequestFactory;
import org.springframework.security.oauth2.provider.code.AuthorizationCodeServices;
import org.springframework.security.oauth2.provider.code.JdbcAuthorizationCodeServices;
import org.springframework.security.oauth2.provider.password.ResourceOwnerPasswordTokenGranter;
import org.springframework.security.oauth2.provider.request.DefaultOAuth2RequestFactory;
import org.springframework.security.oauth2.provider.token.DefaultTokenServices;
import org.springframework.security.oauth2.provider.token.TokenEnhancer;
import org.springframework.security.oauth2.provider.token.TokenStore;
import org.springframework.security.oauth2.provider.token.store.redis.RedisTokenStore;
import org.springframework.security.web.authentication.preauth.PreAuthenticatedAuthenticationProvider;

import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

@Configuration
@EnableAuthorizationServer
public class AuthorizationServerConfigure extends AuthorizationServerConfigurerAdapter {
    private final AuthenticationManager authenticationManager;
    private final UserDetailsService userDetailsService;
    private final ClientDetailsService clientDetailsService;
    private final DataSource dataSource;
    private final RedisConnectionFactory redisConnectionFactory;

    public AuthorizationServerConfigure(AuthenticationManager authenticationManager,
                                        UserDetailsService userDetailsService,
                                        ClientDetailsService clientDetailsService,
                                        DataSource dataSource,
                                        RedisConnectionFactory redisConnectionFactory) {
        this.authenticationManager = authenticationManager;
        this.userDetailsService = userDetailsService;
        this.clientDetailsService = clientDetailsService;
        this.dataSource = dataSource;
        this.redisConnectionFactory = redisConnectionFactory;
    }

    /**
     * oauth2的客户端配置
     *
     * @param clients
     * @throws Exception
     */
    @Override
    public void configure(ClientDetailsServiceConfigurer clients) throws Exception {
       /* clients.inMemory()
                .withClient("client")
                .secret(passwordEncoder.encode("security_secret"))
                .authorizedGrantTypes("refresh_token","password","authorization_code")
                .scopes("all")
                .resourceIds("flow-app")
                .redirectUris("https://www.baidu.com/callback");*/
        clients.withClientDetails(clientDetailsService);
    }

    /**
     * 令牌端点的安全配置
     *
     * @param security
     */
    @Override
    public void configure(AuthorizationServerSecurityConfigurer security) {
        security
                .tokenKeyAccess("permitAll()")
                .checkTokenAccess("permitAll()")
                .allowFormAuthenticationForClients();
    }

    /**
     * 配置授权
     *
     * @param endpoints
     * @throws Exception
     */
    @Override
    public void configure(AuthorizationServerEndpointsConfigurer endpoints) throws Exception {
        endpoints
                //认证管理器
                .authenticationManager(authenticationManager)
                //用户信息
                .userDetailsService(userDetailsService)
                .allowedTokenEndpointRequestMethods(HttpMethod.GET, HttpMethod.POST)
                // 授权码管理策略，针对授权码模式有效，会将授权码放到 auth_code 表，授权后就会删除它
                .authorizationCodeServices(jdbcAuthorizationCodeServices());
        // 使用redis的token方式
        endpoints
                //token存储
                .tokenStore(tokenStore())
                // 添加令牌增强器
                .tokenEnhancer(customTokenEnhancer());


        // 自定义异常转换类(替换oauth2默认返回格式，改成AjaxResponse统一格式)
        // endpoints.exceptionTranslator(responseExceptionTranslator);
    }

    /**
     * 授权码管理策略
     *
     * @return
     */
    @Bean
    public AuthorizationCodeServices jdbcAuthorizationCodeServices() {
        return new JdbcAuthorizationCodeServices(dataSource);
    }

    /**
     * redis存储token
     *
     * @return
     */
    @Bean
    @ConditionalOnMissingBean(value = TokenStore.class)
    public TokenStore tokenStore() {
        RedisTokenStore tokenStore = new RedisTokenStore(redisConnectionFactory);
        tokenStore.setAuthenticationKeyGenerator(new AuthenticationKeyGenerator());
        return tokenStore;
    }

    @Primary
    @Bean
    public DefaultTokenServices tokenServices() {
        DefaultTokenServices tokenServices = new DefaultTokenServices() {
            @Override
            public OAuth2AccessToken createAccessToken(OAuth2Authentication authentication) throws AuthenticationException {
                // 如果用户在其他地方已经登录，就踢下线
                /*OAuth2AccessToken existingAccessToken = tokenStore.getAccessToken(authentication);
                if (existingAccessToken != null) {
                    tokenStore.removeAccessToken(existingAccessToken);
                    if (existingAccessToken.getRefreshToken() != null) {
                        tokenStore.removeRefreshToken(existingAccessToken.getRefreshToken());
                    }
                }*/
                return super.createAccessToken(authentication);
            }
        };
        // 请求回来的token都会变.但是请求的refresh token不会续期(false)或者重置为初始化时间(true)
        tokenServices.setReuseRefreshToken(true);
        // 支持刷新令牌（默认false）
        tokenServices.setSupportRefreshToken(true);
        tokenServices.setClientDetailsService(clientDetailsService);
        // token加强链
        //TokenEnhancerChain tokenEnhancerChain = new TokenEnhancerChain();
        //tokenEnhancerChain.setTokenEnhancers(Lists.newArrayList(tokenEnhancer()));
        tokenServices.setTokenEnhancer(customTokenEnhancer());
        // token存储
        tokenServices.setTokenStore(tokenStore());

        if (userDetailsService != null) {
            PreAuthenticatedAuthenticationProvider provider = new PreAuthenticatedAuthenticationProvider();
            provider.setPreAuthenticatedUserDetailsService(new UserDetailsByNameServiceWrapper<>(userDetailsService));
            ArrayList<AuthenticationProvider> list = new ArrayList<>();
            list.add(provider);
            tokenServices.setAuthenticationManager(new ProviderManager(list));
        }
        return tokenServices;
    }

    /**
     * token增强器
     *
     * @return
     */
    @Bean
    public TokenEnhancer customTokenEnhancer() {
        return new TokenEnhancer() {
            @Override
            public OAuth2AccessToken enhance(OAuth2AccessToken accessToken, OAuth2Authentication authentication) {
                if (accessToken instanceof DefaultOAuth2AccessToken) {
                    // 添加附加信息
                    Map<String, Object> additionalInformation = new HashMap<>();
                    additionalInformation.put("clientId", authentication.getOAuth2Request().getClientId());
                    additionalInformation.put("resourceIds", authentication.getOAuth2Request().getResourceIds());
                    ((DefaultOAuth2AccessToken) accessToken).setAdditionalInformation(additionalInformation);
                }
                return accessToken;
            }
        };
    }

    /**
     * 默认密码模式配置  要不要都行.主要是给社交登录获取该bean使用使用
     *
     * @return
     */
    @Bean
    public ResourceOwnerPasswordTokenGranter resourceOwnerPasswordTokenGranter(OAuth2RequestFactory oAuth2RequestFactory) {
        return new ResourceOwnerPasswordTokenGranter(authenticationManager, tokenServices(), clientDetailsService, oAuth2RequestFactory);
    }

    @Bean
    public DefaultOAuth2RequestFactory oAuth2RequestFactory() {
        return new DefaultOAuth2RequestFactory(clientDetailsService);
    }

}
