package com.flow.common.websocket.configure;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.messaging.simp.config.ChannelRegistration;
import org.springframework.messaging.simp.config.MessageBrokerRegistry;
import org.springframework.messaging.support.ChannelInterceptor;
import org.springframework.scheduling.concurrent.ThreadPoolTaskScheduler;
import org.springframework.web.socket.CloseStatus;
import org.springframework.web.socket.WebSocketHandler;
import org.springframework.web.socket.WebSocketSession;
import org.springframework.web.socket.config.annotation.EnableWebSocketMessageBroker;
import org.springframework.web.socket.config.annotation.StompEndpointRegistry;
import org.springframework.web.socket.config.annotation.WebSocketMessageBrokerConfigurer;
import org.springframework.web.socket.config.annotation.WebSocketTransportRegistration;
import org.springframework.web.socket.handler.WebSocketHandlerDecorator;
import org.springframework.web.socket.handler.WebSocketHandlerDecoratorFactory;

import java.security.Principal;

@Configuration
@EnableWebSocketMessageBroker
public class WebSocketConfigure implements WebSocketMessageBrokerConfigurer {
    @Autowired
    private ChannelInterceptor channelInterceptor;

    /**
     * 这个Bean会自动注册使用@ServerEndpoint注解声明的websocket endpoint
     * @return
     */
    /*@Bean
    public ServerEndpointExporter serverEndpointExporter() {
        return new ServerEndpointExporter();
    }*/

    @Override
    public void configureWebSocketTransport(WebSocketTransportRegistration registry) {
        // 限制入站消息大小：8KB
        registry.setMessageSizeLimit(8 * 1024);
        registry.addDecoratorFactory(new WebSocketHandlerDecoratorFactory(){
            @Override
            public WebSocketHandler decorate(WebSocketHandler handler) {
                return new WebSocketHandlerDecorator(handler){
                    /**
                     * 连接建立后
                     * @param session
                     * @throws Exception
                     */
                    @Override
                    public void afterConnectionEstablished(WebSocketSession session) throws Exception {
                        Principal principal = session.getPrincipal();
                        System.out.println("principal = " + principal);
                        super.afterConnectionEstablished(session);
                    }

                    /**
                     * 连接关闭后
                     * @param session
                     * @param closeStatus
                     * @throws Exception
                     */
                    @Override
                    public void afterConnectionClosed(WebSocketSession session, CloseStatus closeStatus) throws Exception {
                        Principal principal = session.getPrincipal();
                        System.out.println("principal = " + principal);
                        super.afterConnectionClosed(session, closeStatus);
                    }
                };
            }
        });
    }

    @Override
    public void registerStompEndpoints(StompEndpointRegistry registry) {
        registry
                // 注册webSocket端点地址
                .addEndpoint("/socket")
                // 可以跨域
                .setAllowedOrigins("*")
                // 支持sockJs
                .withSockJS();
    }

    /**
     * 注册相关的消息频道
     *
     * @param registry
     */
    @Override
    public void configureMessageBroker(MessageBrokerRegistry registry) {
        ThreadPoolTaskScheduler te = new ThreadPoolTaskScheduler();
        te.setPoolSize(1);
        te.setThreadNamePrefix("wss-heartbeat-thread-");
        te.initialize();
        registry
                // 客户端向服务端发送消息需有/app 前缀
                // .setApplicationDestinationPrefixes("/app")
                // 设置客户端接收点对点消息地址的前缀，默认为 /user
                .setUserDestinationPrefix("/user")
                // 设置两个频道，topic用于广播，queue用于点对点发送
                .enableSimpleBroker("/topic", "/queue")
                // 服务端10s发送一次心跳给客户端，服务端15s未收到心跳则认为连接断开
                .setHeartbeatValue(new long[]{10000, 0}).setTaskScheduler(te);
    }


    /**
     * 加入拦截器主要是为了验证权限的
     *
     * @param registration
     */
    @Override
    public void configureClientInboundChannel(ChannelRegistration registration) {
        // 添加通道拦截器
        registration.interceptors(channelInterceptor);
    }

}
