package com.flow.common.websocket.core;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.Ordered;
import org.springframework.core.annotation.Order;
import org.springframework.messaging.Message;
import org.springframework.messaging.MessageChannel;
import org.springframework.messaging.simp.stomp.StompCommand;
import org.springframework.messaging.simp.stomp.StompHeaderAccessor;
import org.springframework.messaging.support.ChannelInterceptor;
import org.springframework.messaging.support.MessageHeaderAccessor;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.security.oauth2.provider.token.TokenStore;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import java.util.Objects;


/**
 * 客户端入站通道拦截器
 */
@Order(Ordered.HIGHEST_PRECEDENCE + 99)
@Component
public class ClientInboundChannelInterceptor implements ChannelInterceptor {
    private static final Logger log = LoggerFactory.getLogger(ClientInboundChannelInterceptor.class);
    @Autowired
    private TokenStore tokenStore;

    /**
     * 实际消息发送频道之前调用
     *
     * @param message
     * @param channel
     * @return
     */
    @Override
    public Message<?> preSend(Message<?> message, MessageChannel channel) {
        StompHeaderAccessor accessor = MessageHeaderAccessor.getAccessor(message, StompHeaderAccessor.class);
        if (Objects.nonNull(accessor)) {
            StompPrincipal user = (StompPrincipal) accessor.getUser();
            if (Objects.nonNull(user)) {
                OAuth2Authentication auth2Authentication = tokenStore.readAuthentication(user.getToken());
                SecurityContextHolder.getContext().setAuthentication(auth2Authentication);
            }
            if (Objects.nonNull(accessor.getCommand())) {
                if (accessor.getCommand() == StompCommand.CONNECT) {
                    log.info("【preSend】首次连接");
                    String authorization = accessor.getFirstNativeHeader("Authorization");
                    if (!StringUtils.isEmpty(authorization)) {
                        try {
                            String token = authorization.replace("Bearer ", "");
                            OAuth2Authentication oAuth2Authentication = tokenStore.readAuthentication(token);
                            SecurityContextHolder.getContext().setAuthentication(oAuth2Authentication);
                            UserDetails userDetails = (UserDetails) oAuth2Authentication.getPrincipal();
                            // 设置当前访问器的认证用户
                            StompPrincipal principal = new StompPrincipal();
                            principal.setToken(token);
                            principal.setName(userDetails.getUsername());
                            accessor.setUser(principal);
                        } catch (Exception e) {
                            throw new IllegalArgumentException("认证失败,不允许连接!");
                        }
                    } else {
                        throw new IllegalArgumentException("认证失败,不允许连接!");
                    }
                } else if (accessor.getCommand() == StompCommand.DISCONNECT) {
                    log.info("【preSend】断开连接");
                } else if (accessor.getCommand() == StompCommand.SUBSCRIBE) {
                    log.info("【preSend】订阅成功");
                } else if (accessor.getCommand() == StompCommand.SEND) {
                    log.info("【preSend】发送成功");
                } else if (accessor.getCommand() == StompCommand.ERROR) {
                    log.info("【preSend】发送失败");
                }
            }
        }
        return message;
    }

    /**
     * 在消息发送后立刻调用，boolean值参数表示该调用的返回值
     *
     * @param message
     * @param channel
     * @param sent
     */
    @Override
    public void postSend(Message<?> message, MessageChannel channel, boolean sent) {
        StompHeaderAccessor accessor = StompHeaderAccessor.wrap(message);
        if (accessor.getCommand() == StompCommand.CONNECT) {
            log.info("【postSend】连接成功");
        } else if (accessor.getCommand() == StompCommand.DISCONNECT) {
            log.info("【postSend】断开连接");
        } else if (accessor.getCommand() == StompCommand.SUBSCRIBE) {
            log.info("【postSend】订阅成功");
        } else if (accessor.getCommand() == StompCommand.SEND) {
            log.info("【postSend】发送成功");
        } else if (accessor.getCommand() == StompCommand.ERROR) {
            log.info("【postSend】发送失败");
        }
        ChannelInterceptor.super.postSend(message, channel, sent);
    }
}
