package com.flow.model.spreadsheet;

import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.fasterxml.jackson.annotation.JsonAnyGetter;
import com.fasterxml.jackson.annotation.JsonAnySetter;
import com.fasterxml.jackson.annotation.JsonInclude;
import lombok.Data;
import lombok.SneakyThrows;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.apache.poi.xssf.usermodel.XSSFCellStyle;
import org.apache.poi.xssf.usermodel.XSSFColor;
import org.apache.poi.xssf.usermodel.XSSFFont;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

@Data
@JsonInclude(JsonInclude.Include.NON_NULL)
public class SheetCellStyle {
    private SheetBorder border;
    private String align;
    private SheetFont font;
    private String bgcolor;
    private Boolean textwrap;
    private String color;
    private Boolean underline;
    private Boolean strike;
    private String valign;
    private String format;
    protected Map<String, Object> other = new HashMap<>();

    @JsonAnyGetter
    public Map<String, Object> getOther() {
        return other;
    }

    @JsonAnySetter
    public void setOther(String name, Object value) {
        this.other.put(name, value);
    }

    @SneakyThrows
    public CellStyle toStyle(SXSSFWorkbook workbook) {
        XSSFCellStyle cellStyle = (XSSFCellStyle) workbook.createCellStyle();
        XSSFFont textFont = Objects.nonNull(this.font) ?
                this.font.toFont(workbook) :
                (XSSFFont) workbook.createFont();
        cellStyle.setFont(textFont);
        // 背景色
        if (StringUtils.isNotBlank(this.bgcolor)) {
            XSSFColor bgColor = new XSSFColor(java.awt.Color.decode(this.bgcolor), null);
            cellStyle.setFillForegroundColor(bgColor);
            cellStyle.setFillPattern(FillPatternType.SOLID_FOREGROUND);
        }
        // 文字颜色
        if (StringUtils.isNotBlank(this.color)) {
            XSSFColor xssfColor = new XSSFColor(java.awt.Color.decode(this.color), null);
            textFont.setColor(xssfColor);
            cellStyle.setFont(textFont);
        }
        // 自动换行
        if (Objects.nonNull(this.textwrap)) {
            cellStyle.setWrapText(textwrap);
        }
        // 对齐方式
        if (StringUtils.isNotBlank(this.align)) {
            switch (this.align) {
                case "left":
                    cellStyle.setAlignment(HorizontalAlignment.LEFT);
                    break;
                case "center":
                    cellStyle.setAlignment(HorizontalAlignment.CENTER);
                    break;
                case "right":
                    cellStyle.setAlignment(HorizontalAlignment.RIGHT);
                    break;
                default:
                    break;
            }
        }
        // 下划线
        if (Objects.nonNull(this.underline)) {
            textFont.setUnderline(this.underline ? Font.U_SINGLE : Font.U_NONE);
        }
        // 删除线
        if (Objects.nonNull(this.strike)) {
            textFont.setStrikeout(this.strike);
        }
        // 对齐方式
        if (StringUtils.isNotBlank(this.valign)) {
            switch (this.valign) {
                case "top":
                    cellStyle.setVerticalAlignment(VerticalAlignment.TOP);
                    break;
                case "middle":
                    cellStyle.setVerticalAlignment(VerticalAlignment.CENTER);
                    break;
                case "bottom":
                    cellStyle.setVerticalAlignment(VerticalAlignment.BOTTOM);
                    break;
                default:
                    break;
            }
        }
        // 格式
        if (StringUtils.isNotBlank(this.format)) {
            switch (this.format) {
                case "number":
                    DataFormat format = workbook.createDataFormat();
                    cellStyle.setDataFormat(format.getFormat("#,##0.00"));
                    break;
                case "percent":
                    cellStyle.setDataFormat(workbook.createDataFormat().getFormat("0.00%"));
                    break;
                case "text":
                    cellStyle.setDataFormat(workbook.createDataFormat().getFormat("@"));
                    break;
                case "rmb":
                    cellStyle.setDataFormat(workbook.createDataFormat().getFormat("￥#,##0.00"));
                    break;
                case "usd":
                    cellStyle.setDataFormat(workbook.createDataFormat().getFormat("$#,##0.00"));
                    break;
                case "eur":
                    cellStyle.setDataFormat(workbook.createDataFormat().getFormat("€#,##0.00"));
                    break;
                case "date":
                    cellStyle.setDataFormat(workbook.createDataFormat().getFormat("dd/mm/yyyy"));
                    break;
                case "datetime":
                    cellStyle.setDataFormat(workbook.createDataFormat().getFormat("dd/mm/yyyy hh:mm:ss"));
                    break;
                case "duration":
                    cellStyle.setDataFormat(workbook.createDataFormat().getFormat("hh:mm:ss"));
                    break;
                default:
                    break;
            }
        }
        // 边框
        if (Objects.nonNull(this.border)) {
            List<String> top = this.border.getTop();
            if (Objects.nonNull(top)) {
                BorderStyle style = this.toBorderStyle(top.get(0));
                cellStyle.setBorderTop(style);
                XSSFColor xssfColor = new XSSFColor(java.awt.Color.decode(top.get(1)), null);
                cellStyle.setTopBorderColor(xssfColor);
            }
            List<String> bottom = this.border.getBottom();
            if (Objects.nonNull(bottom)) {
                BorderStyle style = this.toBorderStyle(bottom.get(0));
                cellStyle.setBorderBottom(style);
                XSSFColor xssfColor = new XSSFColor(java.awt.Color.decode(bottom.get(1)), null);
                cellStyle.setBottomBorderColor(xssfColor);
            }
            List<String> left = this.border.getLeft();
            if (Objects.nonNull(left)) {
                BorderStyle style = this.toBorderStyle(left.get(0));
                cellStyle.setBorderLeft(style);
                XSSFColor xssfColor = new XSSFColor(java.awt.Color.decode(left.get(1)), null);
                cellStyle.setLeftBorderColor(xssfColor);
            }
            List<String> right = this.border.getRight();
            if (Objects.nonNull(right)) {
                BorderStyle style = this.toBorderStyle(right.get(0));
                cellStyle.setBorderRight(style);
                XSSFColor xssfColor = new XSSFColor(java.awt.Color.decode(right.get(1)), null);
                cellStyle.setRightBorderColor(xssfColor);
            }
        }
        return cellStyle;
    }

    /**
     * 边框样式
     *
     * @param borderStyle
     * @return
     */
    private BorderStyle toBorderStyle(String borderStyle) {
        switch (borderStyle) {
            case "dotted":
                return BorderStyle.DOTTED;
            case "thin":
                return BorderStyle.THIN;
            case "medium":
                return BorderStyle.MEDIUM;
            case "thick":
                return BorderStyle.THICK;
            default:
                return BorderStyle.NONE;
        }
    }
}
