package com.flow.listener;

import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.flow.entity.FlowDefine;
import com.flow.entity.FlowInstance;
import com.flow.entity.FlowTask;
import com.flow.entity.Notify;
import com.flow.entity.settings.CancelConfig;
import com.flow.entity.settings.Settings;
import com.flow.enums.ApprovalNobodyEnum;
import com.flow.enums.AssigneeTypeEnum;
import com.flow.enums.NotifyEnum;
import com.flow.enums.ProcessStatus;
import com.flow.flowable.utils.ProcessElementUtil;
import com.flow.service.FlowDefineService;
import com.flow.service.FlowInstanceService;
import com.flow.service.FlowTaskService;
import com.flow.service.NotifyService;
import lombok.extern.slf4j.Slf4j;
import org.flowable.bpmn.model.FlowElement;
import org.flowable.common.engine.api.delegate.event.FlowableEngineEntityEvent;
import org.flowable.engine.TaskService;
import org.flowable.engine.delegate.DelegateExecution;
import org.flowable.engine.delegate.event.AbstractFlowableEngineEventListener;
import org.flowable.engine.delegate.event.FlowableCancelledEvent;
import org.flowable.engine.delegate.event.FlowableProcessStartedEvent;
import org.flowable.engine.delegate.event.impl.FlowableProcessCancelledEventImpl;
import org.flowable.engine.impl.cfg.ProcessEngineConfigurationImpl;
import org.flowable.engine.impl.persistence.entity.ExecutionEntityImpl;
import org.flowable.task.service.impl.persistence.entity.TaskEntityImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.support.TransactionSynchronization;
import org.springframework.transaction.support.TransactionSynchronizationManager;

import java.time.LocalDateTime;
import java.util.Map;

@Component
@Slf4j
public class GlobalActivityEventListener extends AbstractFlowableEngineEventListener {
    @Autowired
    private FlowDefineService flowDefineService;
    @Autowired
    private FlowTaskService flowTaskService;
    @Autowired
    private TaskService taskService;
    @Autowired
    private NotifyService notifyService;
    @Autowired
    private FlowInstanceService flowInstanceService;
    @Autowired
    private ProcessEngineConfigurationImpl processEngineConfiguration;

    // 任务创建监听器
    @Override
    protected void taskCreated(FlowableEngineEntityEvent event) {
        TaskEntityImpl entity = (TaskEntityImpl) event.getEntity();
        // 更新任务状态
        FlowTask flowTask = FlowTask.build(entity);
        flowTaskService.save(flowTask);
        DelegateExecution execution = super.getExecution(event);
        FlowElement currentFlowElement = execution.getCurrentFlowElement();
        String nobody = ProcessElementUtil.getExtensionElementValue(currentFlowElement, "nobody");
        String assigneeType = ProcessElementUtil.getExtensionElementValue(currentFlowElement, "assigneeType");
        String nobodyUsers = ProcessElementUtil.getExtensionElementValue(currentFlowElement, "nobodyUsers");

        if (StringUtils.isNotBlank(assigneeType)) {
            if (assigneeType.equals(AssigneeTypeEnum.AUTO_PASS.getType())) {
                // 自动通过
                flowTaskService.autoComplete(entity.getId());
            } else if (assigneeType.equals(AssigneeTypeEnum.AUTO_REFUSE.getType())) {
                // 自动拒绝
                flowTaskService.autoRefuse(entity.getId());
            }
            return;
        }
        // 审批人为空方式
        if (StringUtils.isBlank(entity.getAssignee()) || "-000000".equals(entity.getAssignee())) {
            entity.setAssignee("admin");
            taskService.setAssignee(entity.getId(), "admin");
            if (StringUtils.isNotBlank(nobody)) {
                if (nobody.equals(ApprovalNobodyEnum.PASS.getNobody())) {
                    flowTaskService.autoComplete(entity.getId(), " 审批人为空");
                } else if (nobody.equals(ApprovalNobodyEnum.REFUSE.getNobody())) {
                    flowTaskService.autoRefuse(entity.getId(), " 审批人为空");
                } else if (nobody.equals(ApprovalNobodyEnum.ASSIGN.getNobody())) {
                    if (StringUtils.isNotBlank(nobodyUsers)) {
                        String[] split = nobodyUsers.split(",");
                        if(split.length > 0){
                            entity.setAssignee(split[0]);
                        }

                    }
                } else if (nobody.equals(ApprovalNobodyEnum.ADMIN.getNobody())) {
                    DelegateExecution parent = execution.getParent();

                }
                return;
            }
        }
        // 消息通知
        TransactionSynchronizationManager.registerSynchronization(new TransactionSynchronization() {
            @Override
            public void afterCommit() {
                Notify notify = new Notify();
                notify.setSubject("新的审批任务");
                notify.setContent(String.format("您有一条待处理的审批任务【%s】，请及时处理。", entity.getName()));
                notify.setSender("admin");
                notify.setType(NotifyEnum.TODO);
                notify.setReceiver(entity.getAssignee());
                notify.setReceivingTime(LocalDateTime.now());
                notify.setUrl("/flow/todo");
                notifyService.notify(notify);
            }
        });
    }

    // 任务完成监听器
    @Override
    protected void taskCompleted(FlowableEngineEntityEvent event) {
        log.info("任务完成监听器");
    }


    // 流程创建监听器
    @Override
    protected void processStarted(FlowableProcessStartedEvent event) {
        ExecutionEntityImpl entity = (ExecutionEntityImpl) event.getEntity();
        // 更新流程实例状态
        FlowDefine define = flowDefineService.getDefine(entity.getProcessDefinitionId());
        FlowInstance flowInstance = FlowInstance.buildStatus(entity.getProcessInstanceId(), ProcessStatus.RUNNING);
        Settings settings = define.getSettings();
        CancelConfig cancel = settings.getCancel();
        if (cancel.getEnable()) {
            flowInstance.setCancelDays(cancel.getDays());
        } else {
            flowInstance.setCancelDays(0);
        }
        flowInstanceService.save(flowInstance);
    }

    // 流程完成监听器
    @Override
    protected void processCompleted(FlowableEngineEntityEvent event) {
        ExecutionEntityImpl entity = (ExecutionEntityImpl) event.getEntity();
        // 更新流程实例状态
        FlowInstance flowInstance = FlowInstance.buildStatus(entity.getProcessInstanceId(), ProcessStatus.COMPLETED);
        flowInstanceService.updateById(flowInstance);
        // 消息通知
        TransactionSynchronizationManager.registerSynchronization(new TransactionSynchronization() {
            @Override
            public void afterCommit() {
                Notify notify = new Notify();
                notify.setSubject("申请已通过");
                notify.setContent(String.format("您申请的流程《%s》已通过", entity.getName()));
                notify.setSender("admin");
                notify.setType(NotifyEnum.SYSTEM);
                notify.setReceiver(entity.getStartUserId());
                notify.setReceivingTime(LocalDateTime.now());
                notify.setUrl("/flow/instance");
                notifyService.notify(notify);
            }
        });
    }

    // 流程取消监听器
    @Override
    protected void processCancelled(FlowableCancelledEvent event) {
        FlowableProcessCancelledEventImpl flowableProcessCancelledEvent = (FlowableProcessCancelledEventImpl) event;
        DelegateExecution execution = flowableProcessCancelledEvent.getExecution();
        Map<String, Object> variables = execution.getVariables();
        System.out.println("variables = " + variables);
    }

}
