package com.flow.entity.node;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.flow.common.core.util.ApplicationContextUtil;
import com.flow.common.core.util.SecurityContextUtil;
import com.flow.entity.FlowDefine;
import com.flow.entity.User;
import com.flow.enums.ApprovalMultiEnum;
import com.flow.enums.ApprovalNobodyEnum;
import com.flow.enums.ApprovalSameEnum;
import com.flow.enums.AssigneeTypeEnum;
import com.flow.service.UserService;
import com.google.common.collect.Lists;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.flowable.bpmn.model.*;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;

@EqualsAndHashCode(callSuper = true)
@Data
public class ApprovalNode extends AssigneeNode {
    // 表单属性
    private List<FormProperty> formProperties = new ArrayList<>();
    // 操作权限
    private Map<String, Boolean> operations = new LinkedHashMap<>();
    // 多人审批方式
    private ApprovalMultiEnum multi;
    // 多人会签通过百分比
    private BigDecimal multiPercent;
    // 审批人为空时处理方式
    private ApprovalNobodyEnum nobody;
    // 审批人与发起人是同一人时
    private ApprovalSameEnum same;
    // 审批人为空时指定人员
    private List<String> nobodyUsers;
    // 开启审批超时
    private TimerBoundary timeout;
    // 任务监听器
    private List<NodeListener> taskListeners;

    @Override
    public List<FlowElement> convert() {
        ArrayList<FlowElement> elements = Lists.newArrayList();
        // 用户节点
        UserTask userTask = new UserTask();
        userTask.setId(this.getId());
        userTask.setName(this.getName());
        userTask.setExecutionListeners(this.buidEventListener());
        if (CollectionUtils.isNotEmpty(this.taskListeners)) {
            List<FlowableListener> listeners = this.taskListeners.stream().filter(l -> StringUtils.isNotBlank(l.getImplementation())).map(listener -> {
                FlowableListener eventListener = new FlowableListener();
                eventListener.setEvent(listener.getEvent());
                eventListener.setImplementation(listener.getImplementation());
                eventListener.setImplementationType(listener.getImplementationType());
                return eventListener;
            }).collect(Collectors.toList());
            userTask.setTaskListeners(listeners);
        }
        // userTask.setAsynchronous(true);
        // userTask.setFormKey(this.getFormKey());
        // 扩展元素
        ExtensionElement assigneeElement = new ExtensionElement();
        assigneeElement.setName("assigneeType");
        assigneeElement.setNamespace("https://flowable.org/model");
        assigneeElement.setNamespacePrefix("flowable");
        assigneeElement.setElementText(this.getAssigneeType().getType());
        userTask.addExtensionElement(assigneeElement);
        ExtensionElement nobodyElement = new ExtensionElement();
        nobodyElement.setName("nobody");
        nobodyElement.setNamespace("https://flowable.org/model");
        nobodyElement.setNamespacePrefix("flowable");
        nobodyElement.setElementText(this.nobody.getNobody());
        userTask.addExtensionElement(nobodyElement);
        ExtensionElement sameElement = new ExtensionElement();
        sameElement.setName("same");
        sameElement.setNamespace("https://flowable.org/model");
        sameElement.setNamespacePrefix("flowable");
        sameElement.setElementText(this.same.getSame());
        userTask.addExtensionElement(sameElement);
        ExtensionElement nobodyUsersElement = new ExtensionElement();
        nobodyUsersElement.setNamespace("https://flowable.org/model");
        nobodyUsersElement.setName("nobodyUsers");
        if (CollectionUtils.isNotEmpty(this.nobodyUsers)) {
            nobodyUsersElement.setElementText(String.join(",", this.nobodyUsers));
        }
        userTask.addExtensionElement(nobodyUsersElement);
        // 审批人
        MultiInstanceLoopCharacteristics multiInstanceLoopCharacteristics = new MultiInstanceLoopCharacteristics();
        if (this.getMulti() == ApprovalMultiEnum.SEQUENTIAL) {
            multiInstanceLoopCharacteristics.setSequential(true);
        } else if (this.getMulti() == ApprovalMultiEnum.JOINT) {
            multiInstanceLoopCharacteristics.setSequential(false);
            if (Objects.nonNull(this.getMultiPercent()) && this.getMultiPercent().compareTo(BigDecimal.ZERO) > 0) {
                BigDecimal percent = this.getMultiPercent().divide(new BigDecimal(100), 2, RoundingMode.DOWN);
                multiInstanceLoopCharacteristics.setCompletionCondition(String.format("${nrOfCompletedInstances/nrOfInstances >= %s}", percent));
            }
        } else if (this.getMulti() == ApprovalMultiEnum.SINGLE) {
            multiInstanceLoopCharacteristics.setSequential(false);
            multiInstanceLoopCharacteristics.setCompletionCondition("${nrOfCompletedInstances > 0}");
        }
        String variable = String.format("%sItem", this.getId());
        multiInstanceLoopCharacteristics.setElementVariable(variable);
        multiInstanceLoopCharacteristics.setInputDataItem(String.format("${%sCollection}", this.getId()));
        userTask.setLoopCharacteristics(multiInstanceLoopCharacteristics);
        userTask.setAssignee(String.format("${%s}", variable));
        // 添加定时边界事件
        if (Objects.nonNull(timeout) && timeout.getEnabled()) {
            BoundaryEvent boundaryEvent = timeout.convert(userTask);
            elements.add(boundaryEvent);
        }
        elements.add(userTask);
        // 下一个节点的连线
        Node child = this.getChild();
        SequenceFlow sequenceFlow = this.buildSequence(child);
        elements.add(sequenceFlow);
        // 下一个节点
        if (Objects.nonNull(child)) {
            child.setBranchId(this.getBranchId());
            List<FlowElement> flowElements = child.convert();
            elements.addAll(flowElements);
        }
        return elements;
    }

    public List<String> getAssignees(Map<String, Object> values,
                                     FlowDefine flowDefine
    ) {
        List<String> assignees = super.getAssignees(values);
        AssigneeTypeEnum assigneeType = this.getAssigneeType();
        switch (assigneeType) {
            case AUTO_REFUSE:
                assignees.add("admin");
                break;
            case AUTO_PASS:
                assignees.add("admin");
                break;
        }
        String userId = SecurityContextUtil.getUserId();
        UserService userService = ApplicationContextUtil.getBean(UserService.class);
        // 审批人与发起人是同一人时
        int indexOf = assignees.indexOf(userId);
        if (indexOf != -1) {
            if (same == ApprovalSameEnum.SKIP) {
                assignees.remove(userId);
            } else if (same == ApprovalSameEnum.LEADER) {
                User leader = userService.getLeader(userId);
                Optional<User> optional = Optional.ofNullable(leader);
                if (optional.isPresent()) {
                    User user = optional.get();
                    assignees.set(indexOf, user.getUsername());
                } else {
                    assignees.set(indexOf, "-000000");
                }
            } else if (same == ApprovalSameEnum.ORG_LEADER) {
                User orgLeader = userService.getOrgLeader(userId);
                Optional<User> optional = Optional.ofNullable(orgLeader);
                if (optional.isPresent()) {
                    User user = optional.get();
                    assignees.set(indexOf, user.getUsername());
                } else {
                    assignees.set(indexOf, "-000000");
                }
            }
        }
        // 审批人为空时
        if (CollectionUtils.isEmpty(assignees)) {
            ApprovalNobodyEnum nobody = this.getNobody();
            if (nobody == ApprovalNobodyEnum.ASSIGN) {
                assignees.addAll(this.getNobodyUsers());
            } else if (nobody == ApprovalNobodyEnum.ADMIN) {
                assignees.addAll(flowDefine.getAdmin());
            } else {
                assignees.add("-000000");
            }
        }
        return assignees;
    }

}
